/*
    Nibbles 68k v. 4.0 -- Nibbles Snake Game Clone
    Copyright (C) 2000-2001 John David Ratliff
    Based on Stilgar's Nibbles for Fargo
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// C Source File
// Created 7/1/2001; 6:57:05 AM

#include "nibbles.h"

#include <stdio.h>
#include <estack.h>
#include <mem.h>

// the high score file data
static const char *scorefile = "nibblehi";

short int loadHiScores(SCORE *hiscores) {
	short int success = TRUE;
	FILE *f;
	
	// open file for reading and check for errors
	if ((f = fopen(scorefile,"rb")) == NULL) {
		success = FALSE;
	} else {
		// read in the hiscore list and check for error
		if (fread(hiscores,sizeof(SCORE),MAX_HISCORES,f) != MAX_HISCORES) {
			success = FALSE;
		}
		
		fclose(f);
	}
	
	return success;
}

short int saveHiScores(SCORE *hiscores) {
	const char *filetype = "N68K";
	short int success = TRUE;
	FILE *f;
	
	// delete the scorefile before recreating
	unlink(scorefile);
	
	if ((f = fopen(scorefile,"wb")) == NULL) {
		success = FALSE;
	} else {
		if (fwrite(hiscores,sizeof(SCORE),MAX_HISCORES,f) != MAX_HISCORES) {
			success = FALSE;
		}
		
		// write the file tag
		fputc(0,f);
		fputs(filetype,f);
		fputc(0,f);
		fputc(OTH_TAG,f);
	
		fclose(f);
	}
	
	return success;
}

short int createHiScoreList(void) {
	SCORE board[MAX_HISCORES];

	// set all the entries in the table to blank
	memset(board,0,sizeof(SCORE) * MAX_HISCORES);

	return (saveHiScores(board));
}

short int needScoreFile(void) {
	FILE *f = fopen(scorefile,"rb");
	short int needed = FALSE;
	
	// if we cannot open the file, we must need to create it
	if (f == NULL) {
		needed = TRUE;
	} else {
		fclose(f);
	}
	
	return needed;
}
