/*
    Nibbles 68k v. 4.0 -- Nibbles Snake Game Clone
    Copyright (C) 2000-2001 John David Ratliff
    Based on Stilgar's Nibbles for Fargo
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// C Source File
// v3.0 Created 12/10/00; 3:36:43 PM
// v3.2 Modified 3/12/01; 7:05:17 AM
// v4.0 Created 7/1/2001; 6:52:49 AM

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include "nibbles.h"

#include <system.h>
#include <stdlib.h>
#include <graph.h>
#include <kbd.h>

#ifdef _TI89_
short int _ti89;	// create TI-89 version
#endif

#ifdef _TI92_
short int _ti92plus;	// create TI-92+ Version
#endif

// Game Title String
const char *title = "Nibbles 68k v4.0";

short int doIntro(short int *gamespeed, short int *gamedifficulty) {
	short int selector = MENUSTART, done = FALSE, key, options = FALSE, selection = 0;
	short int speed = *gamespeed, difficulty = *gamedifficulty;
	
	// clear the screen
	ClrScr();
	
	// draw the screen borders
	drawBorder();
	
	// draw the logo
	drawLogo();
	
	// draw the menu
	drawMenu();
	
	// draw copyright message
	drawCopyright();
	
	// draw the initial selector
	drawSelector(selector);
	
	while (!done) {
		// wait for keypress
		while ((key = getKey()) == 0);
		delay(KEYDELAY);
		
		if (key == KUP) {
			// move up the menu
			drawSelector(selector);
			
			if (selector == MENUSTART) {
				selector = MENUEND;
			} else {
				selector -= MENUSTEP;
			}
			
			drawSelector(selector);
		} else if (key == KDOWN) {
			// move down the selector
			drawSelector(selector);
			
			if (selector == MENUEND) {
				selector = MENUSTART;
			} else {
				selector += MENUSTEP;
			}
			
			drawSelector(selector);
		} else if (key == KLEFT) {
			if (options) {
				selection = (selector - MENUSTART) / MENUSTEP;
				
				if (selection == START) {
					// same as speed option -- option 1 == SPEED
					drawSpeedOption(speed);
					
					if (speed == VERYSLOW) {
						speed = VERYFAST;
					} else {
						speed--;
					}
					
					drawSpeedOption(speed);
				} else if (selection == OPTIONS) {
					// same as difficulty option -- option 2 == DIFFICULTY
					drawDifficultyOption(difficulty);
					
					if (difficulty == EASY) {
						difficulty = HARD;
					} else {
						difficulty--;
					}
					
					drawDifficultyOption(difficulty);
				}
			}
		} else if (key == KENTER || key == KRIGHT) {
			// select menu option
			selection = (selector - MENUSTART) / MENUSTEP;
			
			if (options) {
				if (selection == QUIT) {
					// close options menu
					options = FALSE;
				
					// switch the options and main menus
					drawOptionsMenu(speed,difficulty);
					drawMenu();
					
					// reset the selector
					drawSelector(selector);
					selector = MENUSTART;
					drawSelector(selector);
				} else if (selection == START) {
					// same as speed option -- option 1 == SPEED
					drawSpeedOption(speed);
					
					if (speed == VERYFAST) {
						speed = VERYSLOW;
					} else {
						speed++;
					}
					
					drawSpeedOption(speed);
				} else if (selection == OPTIONS) {
					// same as difficulty option -- option 2 == DIFFICULTY
					drawDifficultyOption(difficulty);
					
					if (difficulty == HARD) {
						difficulty = EASY;
					} else {
						difficulty++;
					}
					
					drawDifficultyOption(difficulty);
				}
			} else {
				if (selection == START || selection == QUIT) {
					done = TRUE;
				} else {
					// setup options menu
					options = TRUE;
					
					// switch the main and options menus
					drawMenu();
					drawOptionsMenu(speed,difficulty);
					
					// reset the selector
					drawSelector(selector);
					selector = MENUSTART;
					drawSelector(selector);
				}
			}
		} else if (key == KESC) {
			// exit the current menu
			if (options) {
				// close options menu
				options = FALSE;
				
				// switch the options and main menus
				drawOptionsMenu(speed,difficulty);
				drawMenu();
				
				// reset the selector
				drawSelector(selector);
				selector = MENUSTART;
				drawSelector(selector);
			} else {
				selection = QUIT;
				done = TRUE;
			}
		}
	}
	
	// set the game options
	*gamespeed = speed;
	*gamedifficulty = difficulty;
	
	return (selection == START) ? TRUE : FALSE;
}

void _main(void) {
	short int font = FontGetSys(), speed = AVERAGE, difficulty = NORMAL;
	
	// create the hiscore file, if it doesn't exist
	if (needScoreFile()) {
		createHiScoreList();
	}
	
	// dissipate any keystrokes that might have been pushed by the user prior to startup
	delay(KEYDELAY);
	
	// setup medium size font
	FontSetSys(F_6x8);

	// seed the random number generator
	randomize();

	// disable interrupts
	OSSetSR(0x0700);
	
	while (doIntro(&speed,&difficulty)) {
		play(speed,difficulty);
	}
	
	// reenable interrupts before exit
	OSSetSR(0x0000);
	
	// restore old font before exiting (necessary??)
	FontSetSys(font);
}
